@@ -313,7 +313,7 @@ gok_keyboard_add_keys_from_xkb_geom (GokKeyboard *pKeyboard, XkbDescPtr kbd)
 {
 	GokKey *pKey = pKeyboard->pKeyFirst;
 	XkbGeometryPtr geom;
-	int row, col, i, gok_row = 1, gok_col = 0;
+	int row, col, i, rightmost, gok_row = 1, gok_col = 0;
 
 	if (kbd) 
 		geom = kbd->geom;
@@ -352,9 +352,23 @@ gok_keyboard_add_keys_from_xkb_geom (GokKeyboard *pKeyboard, XkbDescPtr kbd)
 					gok_col = pKey->Right;
 				else
 					gok_row = pKey->Bottom;
+				rightmost = ( gok_col > rightmost )? gok_col : rightmost;
 			}
 		}
 	}
+
+	if (gok_main_get_extras())
+	{
+		/* add a branch key for special text edit functions */
+		pKey = gok_key_new (pKey, NULL, pKeyboard);
+		pKey->Style = KEYSTYLE_GENERALDYNAMIC;
+		pKey->Type = KEYTYPE_BRANCHEDITTEXT;
+		pKey->Top = row + 1;
+		pKey->Bottom = row + 2;
+		pKey->Left = 0;
+		pKey->Right = rightmost;
+		gok_key_add_label (pKey, _("Accessible Text Manipulation"),NULL);
+	}
 }
 
 Display *
@@ -374,6 +388,7 @@ gok_keyboard_get_core ()
 {
 	GokKeyboard *pKeyboard = gok_keyboard_new ();
 	GokKey *pKey;
+	gok_log_enter();
 	gok_keyboard_set_name (pKeyboard, _("Keyboard"));
 	pKeyboard->bRequiresLayout = FALSE;
 	pKeyboard->bLaidOut = TRUE;
@@ -392,9 +407,10 @@ gok_keyboard_get_core ()
 	gok_keyboard_add_keys_from_xkb_geom (pKeyboard, 
 					     gok_keyboard_get_xkb_desc ());
 	gok_keyboard_count_rows_columns (pKeyboard);
-	fprintf (stderr, "created core keyboard with %d rows and %d columns\n",
+	gok_log ("created core keyboard with %d rows and %d columns\n",
 		 gok_keyboard_get_number_rows (pKeyboard), 
 		 gok_keyboard_get_number_columns (pKeyboard));
+	gok_log_leave();
 	return pKeyboard;
 }
 
